<?php

# -*- coding: utf-8 -*-

declare(strict_types=1);

namespace Mollie\WooCommerce\Subscription;

use DateTime;
use Inpsyde\Modularity\Module\ExecutableModule;
use Inpsyde\Modularity\Module\ModuleClassNameIdTrait;
use Mollie\WooCommerce\Gateway\MolliePaymentGateway;
use Mollie\WooCommerce\Settings\Settings;
use Mollie\WooCommerce\Shared\Data;
use Psr\Container\ContainerInterface;
use Psr\Log\LoggerInterface as Logger;
use Psr\Log\LogLevel;

class SubscriptionModule implements ExecutableModule
{
    use ModuleClassNameIdTrait;

    /**
     * @var mixed
     */
    protected $logger;
    /**
     * @var mixed
     */
    protected $dataHelper;
    /**
     * @var mixed
     */
    protected $settingsHelper;

    public function run(ContainerInterface $container): bool
    {
        $this->logger = $container->get(Logger::class);
        assert($this->logger instanceof Logger);
        $this->dataHelper = $container->get('settings.data_helper');
        assert($this->dataHelper instanceof Data);
        $this->settingsHelper = $container->get('settings.settings_helper');
        assert($this->settingsHelper instanceof Settings);
        $this->maybeFixSubscriptions();
        $this->schedulePendingPaymentOrdersExpirationCheck();
        return true;
    }

    /**
     * See MOL-322, MOL-405
     */
    public function maybeFixSubscriptions()
    {
        $fixer = new MaybeFixSubscription();
        $fixer->maybeFix();
    }

    /**
     * WCSubscription related.
     */
    public function schedulePendingPaymentOrdersExpirationCheck()
    {
        if (class_exists('WC_Subscriptions_Order')) {
            $settings_helper = $this->settingsHelper;
            $time = $settings_helper->getPaymentConfirmationCheckTime();
            $nextScheduledTime = wp_next_scheduled(
                'pending_payment_confirmation_check'
            );
            if (!$nextScheduledTime) {
                wp_schedule_event(
                    $time,
                    'daily',
                    'pending_payment_confirmation_check'
                );
            }

            add_action(
                'pending_payment_confirmation_check',
                [$this, 'checkPendingPaymentOrdersExpiration']
            );
        }
    }

    /**
     *
     */
    public function checkPendingPaymentOrdersExpiration()
    {
        global $wpdb;
        $currentDate = new DateTime();
        $items = $wpdb->get_results("SELECT * FROM {$wpdb->mollie_pending_payment} WHERE expired_time < {$currentDate->getTimestamp()};");
        foreach ($items as $item) {
            $order = wc_get_order($item->post_id);

            // Check that order actually exists
            if ($order === false) {
                return false;
            }

            if ($order->get_status() === MolliePaymentGateway::STATUS_COMPLETED) {
                $new_order_status = MolliePaymentGateway::STATUS_FAILED;
                $paymentMethodId = $order->get_meta('_payment_method_title', true);
                $molliePaymentId = $order->get_meta('_mollie_payment_id', true);
                $order->add_order_note(sprintf(
                                       /* translators: Placeholder 1: payment method title, placeholder 2: payment ID */
                    __('%1$s payment failed (%2$s).', 'mollie-payments-for-woocommerce'),
                    $paymentMethodId,
                    $molliePaymentId
                ));

                $order->update_status($new_order_status, '');
                if ($order->get_meta('_order_stock_reduced', $single = true)) {
                    // Restore order stock
                    $this->dataHelper->restoreOrderStock($order);

                    $this->logger->log(LogLevel::DEBUG, __METHOD__ . " Stock for order {$order->get_id()} restored.");
                }

                $wpdb->delete(
                    $wpdb->mollie_pending_payment,
                    [
                        'post_id' => $order->get_id(),
                    ]
                );
            }
        }
    }
}
